﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion
using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.Common;
using HeuristicLab.Data;
using HeuristicLab.Data.Views;
using HeuristicLab.MainForm;
using HeuristicLab.MainForm.WindowsForms;

namespace HeuristicLab.Problems.DataAnalysis.Views {
  [View("Estimated Values")]
  [Content(typeof(ITimeSeriesPrognosisSolution))]
  public partial class TimeSeriesPrognosisSolutionEstimatedValuesView : DataAnalysisSolutionEvaluationView {
    private const string TARGETVARIABLE_SERIES_NAME = "Target Variable";
    private const string PROGNOSEDVALUES_TRAINING_SERIES_NAME = "Prognosed Values (training)";
    private const string PROGNOSEDVALUES_TEST_SERIES_NAME = "Prognosed Values (test)";

    public new ITimeSeriesPrognosisSolution Content {
      get { return (ITimeSeriesPrognosisSolution)base.Content; }
      set {
        base.Content = value;
      }
    }

    private StringConvertibleMatrixView matrixView;

    public TimeSeriesPrognosisSolutionEstimatedValuesView()
      : base() {
      InitializeComponent();
      matrixView = new StringConvertibleMatrixView();
      matrixView.ShowRowsAndColumnsTextBox = false;
      matrixView.ShowStatisticalInformation = false;
      matrixView.Dock = DockStyle.Fill;
      this.Controls.Add(matrixView);
    }

    #region events
    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      Content.ModelChanged += new EventHandler(Content_ModelChanged);
      Content.ProblemDataChanged += new EventHandler(Content_ProblemDataChanged);
    }

    protected override void DeregisterContentEvents() {
      base.DeregisterContentEvents();
      Content.ModelChanged -= new EventHandler(Content_ModelChanged);
      Content.ProblemDataChanged -= new EventHandler(Content_ProblemDataChanged);
    }

    private void Content_ProblemDataChanged(object sender, EventArgs e) {
      OnContentChanged();
    }

    private void Content_ModelChanged(object sender, EventArgs e) {
      OnContentChanged();
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      UpdateEstimatedValues();
    }

    private void UpdateEstimatedValues() {
      if (InvokeRequired) {
        Invoke((Action)UpdateEstimatedValues);
        return;
      }
      if (Content == null) return;

      var targetVariable = Content.ProblemData.TargetVariable;
      StringMatrix matrix = null;
      List<string> columnNames = new List<string>();
      columnNames.Add("Id");

      string[,] values = new string[Content.ProblemData.Dataset.Rows, 4];
      var prognosedTraining = Content.GetPrognosedValues(Content.ProblemData.TrainingIndizes.Take(1), Content.ProblemData.TrainingPartition.Size.ToEnumerable()).First();
      var prognosedTest = Content.GetPrognosedValues(Content.ProblemData.TestIndizes.Take(1), Content.ProblemData.TestPartition.Size.ToEnumerable()).First();
      double[] target = Content.ProblemData.Dataset.GetDoubleValues(targetVariable).ToArray();


      foreach (var row in Enumerable.Range(0, Content.ProblemData.Dataset.Rows)) {
        values[row, 0] = row.ToString();
        values[row, 1] = target[row].ToString();
      }

      var rowsEnumerator = Content.ProblemData.TrainingIndizes.GetEnumerator();
      var prognosisEnumerator = prognosedTraining.GetEnumerator();
      while (rowsEnumerator.MoveNext() & prognosisEnumerator.MoveNext()) {
        var row = rowsEnumerator.Current;
        var prognosis = prognosisEnumerator.Current;
        values[row, 2] = prognosis.ToString();
      }

      rowsEnumerator = Content.ProblemData.TestIndizes.GetEnumerator();
      prognosisEnumerator = prognosedTest.GetEnumerator();
      while (rowsEnumerator.MoveNext() & prognosisEnumerator.MoveNext()) {
        var row = rowsEnumerator.Current;
        var prognosis = prognosisEnumerator.Current;
        values[row, 3] = prognosis.ToString();
      }

      columnNames.AddRange(new string[] { targetVariable + "(actual)", targetVariable + "(training)", targetVariable + "(test)" });
      matrix = new StringMatrix(values);
      matrix.ColumnNames = columnNames.ToArray();
      matrix.SortableView = true;
      matrixView.Content = matrix;
    }
    #endregion
  }
}
